#ifndef LIBCRYPTO_COMPAT_H
#define LIBCRYPTO_COMPAT_H

#ifdef NETAPP

#include <openssl/opensslv.h>

#if OPENSSL_VERSION_NUMBER < 0x10100000L

#ifndef _KERNEL
#include <openssl/asn1.h>
#include <openssl/rsa.h>
#include <openssl/dsa.h>
#include <openssl/ecdsa.h>
#include <openssl/dh.h>
#include <openssl/bio.h>
#include <openssl/ts.h>
#include <openssl/x509.h>
#include <openssl/hmac.h>
#endif

#include <openssl/evp.h>

#ifdef  __cplusplus
extern "C" {
#endif

#ifndef _KERNEL

#define ASN1_INTEGER_dup M_ASN1_INTEGER_dup
void ASN1_INTEGER_free(ASN1_INTEGER* asn1);

#define ASN1_TIME_new M_ASN1_TIME_new
#define ASN1_TIME_dup M_ASN1_TIME_dup
void ASN1_TIME_free(ASN1_TIME* asn1);

#define ASN1_IA5STRING_new M_ASN1_IA5STRING_new

int BIO_up_ref(BIO *bio);

#define BIO_set_data(x,v) (x->ptr=v)
#define BIO_get_data(x) x->ptr
#define BIO_set_init(x,v) (x->init=v)

BIO_METHOD *BIO_meth_new(int type, const char *name);
int BIO_meth_set_write(BIO_METHOD *biom,
                       int (*bwrite) (BIO *, const char *, int));
int BIO_meth_set_read(BIO_METHOD *biom,
                      int (*bread) (BIO *, char *, int));
int BIO_meth_set_puts(BIO_METHOD *biom,
                      int (*bputs) (BIO *, const char *));
int BIO_meth_set_gets(BIO_METHOD *biom,
                      int (*bgets) (BIO *, char *, int));
int BIO_meth_set_ctrl(BIO_METHOD *biom,
                      long (*ctrl) (BIO *, int, long, void *));
int BIO_meth_set_create(BIO_METHOD *biom, int (*create) (BIO *));
int BIO_meth_set_destroy(BIO_METHOD *biom, int (*destroy) (BIO *));
int BIO_meth_set_callback_ctrl(BIO_METHOD *biom,
                               long (*callback_ctrl) (BIO *, int,
                                                      bio_info_cb *));
long (*BIO_meth_get_ctrl(BIO_METHOD *biom)) (BIO *, int, long, void *);
#define BIO_meth_free(x) OPENSSL_free(x)

int RSA_set0_key(RSA *r, BIGNUM *n, BIGNUM *e, BIGNUM *d);
int RSA_set0_factors(RSA *r, BIGNUM *p, BIGNUM *q);
int RSA_set0_crt_params(RSA *r, BIGNUM *dmp1, BIGNUM *dmq1, BIGNUM *iqmp);
void RSA_get0_key(const RSA *r, const BIGNUM **n, const BIGNUM **e, const BIGNUM **d);
void RSA_get0_factors(const RSA *r, const BIGNUM **p, const BIGNUM **q);
void RSA_get0_crt_params(const RSA *r, const BIGNUM **dmp1, const BIGNUM **dmq1, const BIGNUM **iqmp);

void DSA_get0_pqg(const DSA *d, const BIGNUM **p, const BIGNUM **q, const BIGNUM **g);
int DSA_set0_pqg(DSA *d, BIGNUM *p, BIGNUM *q, BIGNUM *g);
void DSA_get0_key(const DSA *d, const BIGNUM **pub_key, const BIGNUM **priv_key);
int DSA_set0_key(DSA *d, BIGNUM *pub_key, BIGNUM *priv_key);

int DSA_bits(const DSA *d);

void DSA_SIG_get0(const DSA_SIG *sig, const BIGNUM **pr, const BIGNUM **ps);
int DSA_SIG_set0(DSA_SIG *sig, BIGNUM *r, BIGNUM *s);

void ECDSA_SIG_get0(const ECDSA_SIG *sig, const BIGNUM **pr, const BIGNUM **ps);
int ECDSA_SIG_set0(ECDSA_SIG *sig, BIGNUM *r, BIGNUM *s);

void DH_get0_pqg(const DH *dh, const BIGNUM **p, const BIGNUM **q, const BIGNUM **g);
int DH_set0_pqg(DH *dh, BIGNUM *p, BIGNUM *q, BIGNUM *g);
void DH_get0_key(const DH *dh, const BIGNUM **pub_key, const BIGNUM **priv_key);
int DH_set0_key(DH *dh, BIGNUM *pub_key, BIGNUM *priv_key);
int DH_set_length(DH *dh, long length);
#endif

#define EVP_CIPHER_impl_ctx_size(e) e->ctx_size
#define EVP_CIPHER_meth_get_init(e) e->init
#define EVP_CIPHER_meth_get_cleanup(e) e->cleanup
#define EVP_CIPHER_meth_get_set_asn1_params(e) e->set_asn1_parameters
#define EVP_CIPHER_meth_get_get_asn1_params(e) e->get_asn1_parameters
#define EVP_CIPHER_meth_get_ctrl(e) e->ctrl
EVP_CIPHER *EVP_CIPHER_meth_new(int cipher_type, int block_size, int key_len);
int EVP_CIPHER_meth_set_iv_length(EVP_CIPHER *cipher, int iv_len);
int EVP_CIPHER_meth_set_flags(EVP_CIPHER *cipher, unsigned long flags);
int EVP_CIPHER_meth_set_impl_ctx_size(EVP_CIPHER *cipher, int ctx_size);
int EVP_CIPHER_meth_set_init(EVP_CIPHER *cipher,
                             int (*init) (EVP_CIPHER_CTX *ctx,
                                          const unsigned char *key,
                                          const unsigned char *iv,
                                          int enc));
int EVP_CIPHER_meth_set_do_cipher(EVP_CIPHER *cipher,
                                  int (*do_cipher) (EVP_CIPHER_CTX *ctx,
                                                    unsigned char *out,
                                                    const unsigned char *in,
                                                    size_t inl));
int EVP_CIPHER_meth_set_cleanup(EVP_CIPHER *cipher,
                                int (*cleanup) (EVP_CIPHER_CTX *));
int EVP_CIPHER_meth_set_set_asn1_params(EVP_CIPHER *cipher,
                                        int (*set_asn1_parameters) (EVP_CIPHER_CTX *,
                                                                    ASN1_TYPE *));
int EVP_CIPHER_meth_set_get_asn1_params(EVP_CIPHER *cipher,
                                        int (*get_asn1_parameters) (EVP_CIPHER_CTX *,
                                                                    ASN1_TYPE *));
int EVP_CIPHER_meth_set_ctrl(EVP_CIPHER *cipher,
                             int (*ctrl) (EVP_CIPHER_CTX *, int type,
                                          int arg, void *ptr));

const unsigned char *EVP_CIPHER_CTX_iv(const EVP_CIPHER_CTX *ctx);
unsigned char *EVP_CIPHER_CTX_iv_noconst(EVP_CIPHER_CTX *ctx);
int EVP_CIPHER_CTX_encrypting(const EVP_CIPHER_CTX *ctx);
#define EVP_CIPHER_CTX_get_cipher_data(ctx) ctx->cipher_data

EVP_MD_CTX *EVP_MD_CTX_new(void);
void EVP_MD_CTX_free(EVP_MD_CTX *ctx);
#define EVP_MD_CTX_reset EVP_MD_CTX_cleanup

#ifndef _KERNEL
HMAC_CTX *HMAC_CTX_new(void);
void HMAC_CTX_free(HMAC_CTX *ctx);
RSA_METHOD *RSA_meth_new(const char *name, int flags);
RSA_METHOD *RSA_meth_dup(const RSA_METHOD *meth);
int RSA_meth_set1_name(RSA_METHOD *meth, const char *name);
#define RSA_meth_get_finish(meth) meth->finish
#define RSA_meth_get_flags(meth) meth->flags
int RSA_meth_set_priv_enc(RSA_METHOD *meth, int (*priv_enc) (int flen, const unsigned char *from, unsigned char *to, RSA *rsa, int padding));
int RSA_meth_set_priv_dec(RSA_METHOD *meth, int (*priv_dec) (int flen, const unsigned char *from, unsigned char *to, RSA *rsa, int padding));
int RSA_meth_set_finish(RSA_METHOD *meth, int (*finish) (RSA *rsa));
void RSA_meth_free(RSA_METHOD *meth);

int RSA_bits(const RSA *r);

RSA *EVP_PKEY_get0_RSA(EVP_PKEY *pkey);
DSA *EVP_PKEY_get0_DSA(EVP_PKEY *pkey);
int EVP_PKEY_up_ref(EVP_PKEY *pkey);
#define EVP_PKEY_base_id(pkey) pkey->type

int X509_up_ref(X509 *x);

int X509_STORE_up_ref(X509_STORE *store);

void X509_STORE_CTX_set0_verified_chain(X509_STORE_CTX *ctx, STACK_OF(X509) *sk);
void X509_STORE_CTX_set_current_cert(X509_STORE_CTX *ctx, X509 *x);

#define X509_CRL_set1_lastUpdate X509_CRL_set_lastUpdate
#define X509_CRL_set1_nextUpdate X509_CRL_set_nextUpdate

#define X509_NAME_ENTRY_set(entry) entry->set

int TS_VERIFY_CTX_set_flags(TS_VERIFY_CTX *ctx, int f);
BIO *TS_VERIFY_CTX_set_data(TS_VERIFY_CTX *ctx, BIO *b);
X509_STORE *TS_VERIFY_CTX_set_store(TS_VERIFY_CTX *ctx, X509_STORE *s);
STACK_OF(X509) *TS_VERIFY_CTS_set_certs(TS_VERIFY_CTX *ctx, STACK_OF(X509) *certs);

#define TS_STATUS_INFO_set_status(a, i) ASN1_INTEGER_set(a->status, i)
#endif

#ifdef  __cplusplus
}
#endif

#endif /* OPENSSL_VERSION_NUMBER */

#endif /* NETAPP */

#endif /* LIBCRYPTO_COMPAT_H */
